@@ -3,6 +3,9 @@ library(Matrix)
 
 source(system.file("test-tools.R", package = "Matrix"))
 
+## from ../R/Auxiliaries.R :
+no_facts <- Matrix:::.drop.factors
+
 ## the empty ones:
 checkMatrix(new("dgeMatrix"))
 checkMatrix(Matrix(,0,0))
@@ -15,10 +18,10 @@ assertError( new("dgeMatrix", Dim = as.integer(c(2,2)), x= as.double(1:5)))
 
 checkMatrix(m1 <- Matrix(1:6, ncol=2))
 checkMatrix(m2 <- Matrix(1:7 +0, ncol=3)) # a (desired) warning
-stopifnot(all(match(is(m1), c("dgeMatrix", "ddenseMatrix", "generalMatrix",
-	    "dMatrix", "denseMatrix", "Matrix", "compMatrix"),0) > 0),
-	  dim(t(m1)) == 2:3,
-	  identical(m1, t(t(m1))))
+c("dgeMatrix", "ddenseMatrix", "generalMatrix", "geMatrix", "dMatrix",
+  "denseMatrix", "compMatrix", "Matrix", "xMatrix", "mMatrix") -> m1.cl
+stopifnot(!anyNA(match(m1.cl, is(m1))),
+	  dim(t(m1)) == 2:3, identical(m1, t(t(m1))))
 c.nam <- paste("C",1:2, sep='')
 dimnames(m1) <- list(NULL, c.nam)
 checkMatrix(m1) # failed in 0.999375-10
@@ -28,34 +31,34 @@ stopifnot(colnames(m1) == c.nam,
 	  identical(m1, t(tm1)))
 
 ## an example of *named* dimnames
-(t34N <- as(unclass(table(x = gl(3,4), y=gl(4,3))), "dgeMatrix"))
+(t34N <- as(unclass(table(x = gl(3,4), y=gl(4,3))), "unpackedMatrix"))
 stopifnot(identical(dimnames(t34N),
 		    dimnames(as(t34N, "matrix"))),
           identical(t34N, t(t(t34N))))
 
 ## "dpo"
 checkMatrix(cm <- crossprod(m1))
-checkMatrix(cp <- as(cm, "dppMatrix"))# 'dpp' + factors
+checkMatrix(cp <- as(cm, "packedMatrix"))# 'dpp' + factors
 checkMatrix(cs <- as(cm, "dsyMatrix"))# 'dsy' + factors
-checkMatrix(dcm <- as(cm, "dgeMatrix"))#'dge'
+checkMatrix(dcm <- as(cm, "generalMatrix"))#'dge'
 checkMatrix(mcm <- as(cm, "dMatrix")) # 'dsy' + factors -- buglet? rather == cm?
 checkMatrix(mc. <- as(cm, "Matrix"))  # dpo --> dsy -- (as above)  FIXME? ??
 stopifnot(identical(mc., mcm),
-	  identical(cm, (2*cm)/2),# remains dpo
+	  identical(no_facts(cm), (2*cm)/2),# remains dpo
 	  identical(cm + cp, cp + cs),# dge
 	  identical(mc., mcm),
 	  all(2*cm == mcm * 2))
 
 checkMatrix(eq <- cm == cs)
 stopifnot(all(eq@x),
-	  identical3(eq, cs == cp, cm == cp),
+	  identical3(pack(eq), cs == cp, cm == cp),
 	  as.logical(!(cs < cp)),
 	  identical4(!(cs < cp), !(cp > cs), cp <= cs, cs >= cp))
 
 ## Coercion to 'dpo' should give an error if result would be invalid
 M <- Matrix(diag(4) - 1)
 assertError(as(M, "dpoMatrix"))
-M. <- as(M, "dgeMatrix")
+M. <- as(M, "generalMatrix")
 M.[1,2] <- 10 # -> not even symmetric anymore
 assertError(as(M., "dpoMatrix"))
 
@@ -63,7 +66,7 @@ assertError(as(M., "dpoMatrix"))
 ## Cholesky
 checkMatrix(ch <- chol(cm))
 checkMatrix(ch2 <- chol(as(cm, "dsyMatrix")))
-checkMatrix(ch3 <- chol(as(cm, "dgeMatrix")))
+checkMatrix(ch3 <- chol(as(cm, "generalMatrix")))
 stopifnot(is.all.equal3(as(ch, "matrix"), as(ch2, "matrix"), as(ch3, "matrix")))
 ### Very basic	triangular matrix stuff
 
@@ -86,22 +89,26 @@ assertError(new("dtrMatrix", Dim = 2:3,
 
 n <- 3:3
 assertError(new("dtCMatrix", Dim = c(n,n), diag = "U"))
-stopifnot(validObject(T <- new("dtTMatrix", Dim = c(n,n), diag = "U")),
-	  identical(as.mat(T), diag(n)),
-	  validObject(M <- new("dtCMatrix", Dim = c(n,n), diag = "U",
-			       p = rep.int(0:0, n+1)))
-	  )
+validObject(T <- new("dtTMatrix", Dim = c(n,n), diag = "U"))
+validObject(M <- new("dtCMatrix", Dim = c(n,n), diag = "U",
+		     p = rep.int(0:0, n+1)))
+stopifnot(identical(as.mat(T), diag(n)))
 
-set.seed(3) ; (p9 <- as(sample(9), "pMatrix"))
-## Check that the correct error message is triggered
-ind.try <- try(p9[1,1] <- 1, silent = TRUE)
-stopifnot(grep("replacing.*sensible", ind.try[1]) == 1,
-	  is.logical(p9[1,]),
+suppressWarnings(RNGversion("3.5.0")); set.seed(3)
+(p9 <- p9. <- as(sample(9), "pMatrix"))
+(np9 <- np9. <- as(p9, "TsparseMatrix"))
+p9.[1,1] <- np9.[1,1] <- TRUE
+stopifnot(is.logical(p9[1,]),
+	  is(p9[2,, drop=FALSE], "indMatrix"),
+	  is(p9[9:1,], "indMatrix"),
 	  isTRUE(p9[-c(1:6, 8:9), 1]),
 	  identical(t(p9), solve(p9)),
-	  identical(p9[TRUE,], as(p9, "ngTMatrix")),
+	  identical(p9[TRUE, ], p9),
+          all.equal(p9[, TRUE], np9), # currently...
+          identical(p9., np9.),
 	  identical(as(diag(9), "pMatrix"), as(1:9, "pMatrix"))
 	  )
+assert.EQ.mat(p9[TRUE,], as.matrix(np9))
 
 ## validObject --> Cparse_validate(.)
 mm <- new("dgCMatrix", Dim = c(3L, 5L),
@@ -114,25 +121,25 @@ m. <- mm
 ip <- c(1:2, 4:3, 6:5) # permute the 'i' and 'x' slot just "inside column":
 m.@i <- m.i <- mm@i[ip]
 m.@x <- m.x <- mm@x[ip]
-stopifnot(grep("row indices are not", validObject(m., test=TRUE)) == 1)
-## 	  validObject(m.))   ## <<-- this auto-sorts  m.
-## stopifnot(identical(mm, m.)) ## since it was auto-sorted
-## and m.@i, m.@x now differ from m.i & m.x respectively:
-## stopifnot(identical(m.i, m.@i[ip]),
-## 	  identical(m.x, m.@x[ip]))
-##
+stopifnot(identical(1L, grep("not increasing within columns",
+                             validObject(m., test = TRUE))))
+Matrix:::.sortCsparse(m.) # don't use this at home, boys!
+m. # now is fixed
+
 ## Make sure that validObject() objects...
 ## 1) to wrong 'p'
 m. <- mm; m.@p[1] <- 1L
-stopifnot(grep("first element of slot p", validObject(m., test=TRUE)) == 1)
+stopifnot(identical(1L, grep("first element of 'p' slot",
+                             validObject(m., test = TRUE))))
 m.@p <- mm@p[c(1,3:2,4:6)]
-stopifnot(grep("^slot p.* non-decreasing", validObject(m., test=TRUE)) == 1)
+stopifnot(identical(1L, grep("not nondecreasing",
+                             validObject(m., test = TRUE))))
 ## 2) to non-strictly increasing i's:
 m. <- mm ; ix <- c(1:3,3,5:6)
 m.@i <- mm@i[ix]
 m.@x <- mm@x[ix]
-stopifnot(identical(grep("slot i is not.* increasing .*column$",
-                         validObject(m., test=TRUE)), 1L))
+stopifnot(identical(1L, grep("not increasing within columns",
+                             validObject(m., test = TRUE))))
 ## ix <- c(1:3, 3:6) # now the the (i,x) slots are too large (and decreasing at end)
 ## m.@i <- mm@i[ix]
 ## m.@x <- mm@x[ix]
@@ -145,7 +152,7 @@ stopifnot(identical(grep("slot i is not.* increasing .*column$",
 m. <- mm
 m.@i <- c(mm@i, NA, NA, NA)
 m.@x <- c(mm@x, 10:12)
-stopifnot(validObject(m.))
+validObject(m.)
 m. # show() now works
 stopifnot(all(m. == mm), # in spite of
 	  length(m.@i) > length(mm@i),
